@@ -1,5 +1,6 @@
 from utils_ak.properties import *
 from utils_ak.architecture import delistify
+from collections import defaultdict
 
 
 class Block:
@@ -10,6 +11,7 @@ class Block:
         props_formatters=None,
         props_accumulators=None,
         props_required_keys=None,
+        props_cache_keys=None,
         **props,
     ):
         block_class = block_class or default_block_class
@@ -17,12 +19,14 @@ class Block:
 
         self.parent = None
         self.children = []
+        self.children_by_cls = defaultdict(list)
 
         self.props = DynamicProps(
             props=props,
             formatters=props_formatters,
             accumulators=props_accumulators,
             required_keys=props_required_keys,
+            cache_keys=props_cache_keys,
         )
 
     def __getitem__(self, item):
@@ -38,7 +42,8 @@ class Block:
 
     def get(self, item):
         if isinstance(item, str):
-            res = [b for b in self.children if b.props["cls"] == item]
+            res = self.children_by_cls[item]
+            # res = [b for b in self.children if b.props["cls"] == item]
         elif isinstance(item, int):
             res = self.children[item]
         elif isinstance(item, slice):
@@ -79,33 +84,19 @@ class Block:
             for b in child.iter(**query):
                 yield b
 
-    # todo: deprecated, remove dependencies
-    def set_parent(self, parent):
-        self.parent = parent
-        self.props.parent = parent.props
-
-    # todo: deprecated, remove dependencies
     def add_child(self, block):
-        block.set_parent(self)
+        block.parent = self
         self.children.append(block)
-        self.props.children.append(block.props)
+        self.children_by_cls[block.props["cls"]].append(block)
+        self.props.add_child(block.props)
         return block
 
-    def connect(self, parent):
-        if parent == self.parent:
-            return
-        if not self.parent:
-            self.disconnect()
-
-        self.parent = parent
-        self.props.parent = parent.props
-
-    def disconnect(self):
-        self.parent.children.remove(self)
-        self.parent.props.children.remove(self.props)
-
-        self.parent = None
-        self.props.parent = None
+    def remove_child(self, block):
+        block.parent = None
+        self.children.remove(block)
+        self.children_by_cls[block.props["cls"]].remove(block)
+        self.props.remove_child(block.props)
+        return block
 
 
 def test_block():
