@@ -29,14 +29,18 @@ class ParallelepipedBlock(Block):
         if "size" not in props:
             props["size"] = SimpleVector(n_dims)
 
+        self.size_cached = None
+
         super().__init__(
             block_class,
             props_formatters={
-                "x": lambda k, v: SimpleVector(v),
-                "size": lambda k, v: SimpleVector(v),
+                "x": lambda k, v: SimpleVector([int(v[0]), int(v[1])]),
+                "size": lambda k, v: SimpleVector([int(v[0]), int(v[1])]),
             },
+            # props_cache_keys=["x"],
             **props,
         )
+
         self.props.accumulators["x"] = x_cumsum_acc
         self.props.accumulators[
             "x_rel"
@@ -51,10 +55,76 @@ class ParallelepipedBlock(Block):
     def x(self):
         return self.props["x"]
 
+    @property
+    def x_rel(self):
+        return self.props["x_rel"]
+
     @property
     def y(self):
         return self.x + self.size
 
+    @property
+    def y_rel(self):
+        return self.x_rel + self.size
+
+    def add_child(self, block):
+        super().add_child(block)
+        self.size_cached = None
+
+    def remove_child(self, block):
+        super().remove_child(block)
+        self.size_cached = None
+
+    @property
+    def size(self):
+        if not self.size_cached:
+            size = self.props["size"]
+            values = []
+            for axis in range(self.n_dims):
+                if size[axis] == 0:
+                    if not self.children:
+                        values.append(0)
+                    else:
+                        start = min([c.x_rel[axis] for c in self.children] + [0])
+                        values.append(
+                            max([c.y_rel[axis] - start for c in self.children])
+                        )
+                else:
+                    values.append(size[axis])
+            self.size_cached = SimpleVector(values)
+        return self.size_cached
+
+    def to_dict(self, props=None, with_children=True):
+        props = props or []
+
+        res = {}
+        res["cls"] = self.props["cls"]
+
+        res["props"] = {}
+
+        for prop in props:
+            if isinstance(prop, str):
+                res["props"][prop] = self.props.get(prop)
+
+            elif isinstance(prop, dict):
+                if "cls" not in prop or (
+                    "cls" in prop and self.props["cls"] == prop["cls"]
+                ):
+                    key = prop["key"]
+                    value = prop.get("value", key)
+                    if isinstance(value, str):
+                        res["props"][key] = self.props[key]
+                    elif callable(value):
+                        res["props"][key] = value(self)
+                    else:
+                        raise Exception("Value should be either callable or string")
+
+        if with_children:
+            res["children"] = [
+                child.to_dict(props, with_children) for child in self.children
+            ]
+        return res
+
     def __str__(self):
         res = (
             self.props["cls"]
@@ -88,22 +158,6 @@ class ParallelepipedBlock(Block):
                 res += "\n"
         return res
 
-    @property
-    def size(self):
-        size = self.props["size"]
-        values = []
-        for axis in range(self.n_dims):
-            if size[axis] == 0:
-                if not self.children:
-                    values.append(0)
-                else:
-                    start = min([c.x[axis] for c in self.children] + [self.x[axis]])
-                    # start = self.x[axis]
-                    values.append(max([c.y[axis] - start for c in self.children]))
-            else:
-                values.append(size[axis])
-        return SimpleVector(values)
-
 
 def test_parallelepiped_block():
     a = ParallelepipedBlock("a", n_dims=2, x=[1, 2])
@@ -124,6 +178,9 @@ def test_parallelepiped_block():
 
     print(a.__repr__())
 
+    print(a.to_dict())
+    print(a.to_dict(["x", {"key": "size", "value": lambda b: list(b.props["size"])}]))
+
 
 if __name__ == "__main__":
     test_parallelepiped_block()
