@@ -1,5 +1,6 @@
 from mongoengine import *
 
+
 from datetime import datetime
 
 
@@ -13,11 +14,13 @@ class Job(Document):
     locked_by = ReferenceField("Worker")
     locked_at = DateTimeField()
 
-    runnable = DictField(required=True)
+    runnable = DictField(required=True)  # docker image name, python main path, ...
     params = DictField()
 
-    running_timeout = IntField()
-    initializing_timeout = IntField(default=20.0, required=True)
+    running_timeout = IntField()  # how much time allowed for the whole job runtime.
+    initializing_timeout = IntField(
+        default=20.0, required=True
+    )  # how much time allowed for job initialization.
 
     status = StringField(
         required=True,
