@@ -4,17 +4,18 @@ import numpy as np
 import os
 import base64
 
-from PIL import Image
 from io import BytesIO
 
 from utils_ak.os import *
-from utils_ak.builtin import iter_get, remove_neighbor_duplicates
+from utils_ak.builtin import iter_get, remove_neighbor_duplicates, delistify
+from utils_ak.iteration import *
 
 pd.set_option("display.max_colwidth", None)
 
 
 def display_with_image_thumbnails(df, shape=None):
     from IPython.display import HTML, display
+    from PIL import Image
 
     def _get_thumbnail(path, shape=None):
         if shape is None:
@@ -137,30 +138,30 @@ def merge_by_columns(dfs):
     return res_df
 
 
-def merge(dfs, by=None, by_index=False, keep="last", sort_index=True):
+def merge(dfs, by, keep="last", sort_index=True):
     """
     :param dfs: list(`pd.DataFrame`)
     :param by: name of column or list of columns names. 'all' for all columns. 'columns' for merge_by_columns method
-    :param by_index: include index or not (bool value)
     :param keep: 'last' or 'first'
 
     :return:
     """
-    if not by and not by_index:
-        raise TypeError("Either by or index should by non-trivial")
+    if not by:
+        return pd.concat(dfs, axis=0)
 
-    if by is None:
-        by = []
-    elif by == "all":
-        pass
-    elif by == "columns":
-        return merge_by_columns(dfs)
-    elif isinstance(by, str):
+    if isinstance(by, str):
         by = [by]
-    elif isinstance(by, list):
-        pass
-    else:
-        raise TypeError("By should be list, str or None")
+
+    by_index = False
+
+    if isinstance(by, list) and "index" in by:
+        by.remove("index")
+        by_index = True
+        by = delistify(by)
+
+    if by == "columns":
+        assert not by_index, "Merging by columns only"
+        return merge_by_columns(dfs)
 
     df = pd.concat(dfs, axis=0)
 
@@ -192,7 +193,12 @@ def pd_read(fn, index_column=None, **kwargs):
     ext = os.path.splitext(fn)[-1]
     if ".zip" in ext:
         ext = os.path.splitext(fn[:-4])[-1]
-    df = getattr(pd, f"read_{ext[1:]}")(fn, **kwargs)
+    if ext == ".msgpack":
+        from mbf_pandas_msgpack import read_msgpack
+
+        df = read_msgpack(fn)
+    else:
+        df = getattr(pd, f"read_{ext[1:]}")(fn, **kwargs)
     if index_column:
         df = df.set_index(index_column)
     return df
@@ -208,10 +214,20 @@ def pd_write(df, fn, index_column=None, **kwargs):
     if ".zip" in ext:
         ext = os.path.splitext(fn[:-4])[-1]
         kwargs["compression"] = "zip"
+
     kwargs["index"] = False
 
+    if ext == ".hdf" and "key" not in kwargs:
+        kwargs["key"] = "key"
+
     tmp_fn = fn + ".tmp"
-    res = getattr(df, f"to_{ext[1:]}")(tmp_fn, **kwargs)
+
+    if ext == ".msgpack":
+        from mbf_pandas_msgpack import to_msgpack
+
+        res = to_msgpack(tmp_fn, df)
+    else:
+        res = getattr(df, f"to_{ext[1:]}")(tmp_fn, **kwargs)
     if os.path.exists(fn):
         remove_path(fn)
     rename_path(tmp_fn, fn)
@@ -250,8 +266,16 @@ def df_to_tree(df, recursive=True, delistify=False):
     return res
 
 
-def df_to_ordered_tree(df, recursive=True, prune_last=True):
+def df_to_ordered_tree(df, column=None, recursive=True, prune_last=True):
     df = df.copy()
+    column = column or df.columns[0]
+    assert column in df.columns
+
+    if column != df.columns[0]:
+        other_columns = list(df.columns)
+        other_columns.remove(column)
+        df = df[[column] + other_columns]
+
     if len(df.columns) == 1:
         res = df[df.columns[0]].tolist()
         if prune_last:
@@ -261,11 +285,9 @@ def df_to_ordered_tree(df, recursive=True, prune_last=True):
     res = []
     pre_res = []
 
-    col = df.columns[0]
-    mark_consecutive_groups(df, col, "__key")
-
-    for value, grp in df.groupby([col, "__key"]):
-        grp.pop(col)
+    mark_consecutive_groups(df, column, "__key")
+    for value, grp in df.groupby([column, "__key"]):
+        grp.pop(column)
         grp.pop("__key")
         if recursive:
             child = df_to_ordered_tree(grp, recursive=True, prune_last=prune_last)
@@ -276,15 +298,93 @@ def df_to_ordered_tree(df, recursive=True, prune_last=True):
     pre_res = list(
         sorted(
             pre_res,
-            key=lambda obj: df[(df[col] == obj[0]) & (df["__key"] == obj[1])].iloc[0][
-                "__key"
-            ],
+            key=lambda obj: df[(df[column] == obj[0]) & (df["__key"] == obj[1])].iloc[
+                0
+            ]["__key"],
         )
     )
     return [(x[0], x[2]) for x in pre_res]
 
 
-def test():
+def crop_invalid_edges(df, side="all"):
+    if side in ["prefix", "all"]:
+        first_idx = df.first_valid_index()
+    else:
+        first_idx = None
+
+    if side in ["suffix", "all"]:
+        last_idx = df.last_valid_index()
+    else:
+        last_idx = None
+    return df.loc[first_idx:last_idx]
+
+
+# todo: better naming
+def split_into_sum_groups(df, values, column, group_column):
+    df = df.copy()
+    assert abs(sum(values) - df[column].sum()) < 1e-5
+
+    values_iterator = SimpleIterator(values)
+    rows_iterator = iter(iter_pairs(list(df.iterrows()), "any_suffix"))
+
+    cur_needed = next(values_iterator)
+    cur_row_pair, next_row_pair = next(rows_iterator)
+    cur_available = cur_row_pair[-1][column]
+
+    res = []
+    i = 0
+    while True:
+        cur_value = min(cur_needed, cur_available)
+        cur_needed -= cur_value
+        cur_available -= cur_value
+
+        row = pd.Series(cur_row_pair[-1])
+        row[column] = cur_value
+        row[group_column] = i
+        res.append(dict(row))
+
+        if cur_needed < 1e-5:
+            if cur_available < 1e-5 and next_row_pair is None:
+                # finished
+                break
+            else:
+                # go to next value
+                cur_needed = next(values_iterator)
+                i += 1
+
+        if cur_available < 1e-5:
+            # go to next row
+            cur_row_pair, next_row_pair = next(rows_iterator)
+            cur_available = cur_row_pair[-1][column]
+
+    df = pd.DataFrame(res)
+    df[group_column] = df[group_column].astype(int)
+    return df
+
+
+def loc_non_inclusive(df, slice):
+    df = df.loc[slice]
+    mask = df.index.where(df.index != slice.stop)
+    mask = pd.Series(mask, index=df.index)
+    mask = mask.fillna(method="bfill")
+    mask = ~mask.isnull()
+
+    if not mask.any():
+        return pd.DataFrame()
+
+    df = df[mask]
+    return df
+
+
+def test_loc_non_inclusive():
+    df = pd.DataFrame(dict(A=range(4)), index=[1, 2, 2, 3])
+    print(loc_non_inclusive(df, slice(None, 1, None)))
+    print(loc_non_inclusive(df, slice(None, 2, None)))
+    print(loc_non_inclusive(df, slice(None, 3, None)))
+    print(loc_non_inclusive(df, slice(None, 4, None)))
+
+
+def test_merge():
     df1 = pd.DataFrame.from_dict({"a": [1, 2, 3], "b": [4, 5, 6], "c": [1, 1, 1]})
     df1 = df1.set_index("c")
     df2 = pd.DataFrame.from_dict({"a": [3, 4, 5], "b": [7, 8, 9]})
@@ -297,22 +397,16 @@ def test():
 
     dfs = [df1, df2]
 
-    print(merge(dfs, by_index=False, by="a"))
-    print()
-    print(merge(dfs, by_index=True, by="a"))
-    print()
-    print(merge(dfs, by_index=True, by=None))
-    print()
-    print(merge(dfs, by_index=True, by=None, keep="first"))
-    print()
-    print(merge(dfs, by_index=True, by="all", keep="first"))
-
-    try:
-        print(merge(dfs, by_index=False, by=None))
-    except Exception as e:
-        print(e)
-    else:
-        raise Exception("Should not happen")
+    print("Merge by a")
+    print(merge(dfs, by="a"))
+    print("Merge by index and a")
+    print(merge(dfs, by=["index", "a"]))
+    print("Merge by index")
+    print(merge(dfs, by="index"))
+    print("Merge by index, keep first")
+    print(merge(dfs, by="index", keep="first"))
+    print("Merge by all and index, keep first")
+    print(merge(dfs, by=["all", "index"], keep="first"))
 
 
 def test_tree():
@@ -335,6 +429,50 @@ def test_tree():
     print(df_to_ordered_tree(df, prune_last=False))
 
 
+def test_read_write():
+    from datetime import datetime, timedelta
+
+    df = pd.DataFrame.from_dict(
+        {"a": [datetime(2020, 1, 1)] * 3, "b": [4, 5, 6], "c": [1, 1, 1]}
+    )
+
+    for fn in ["tmp.parquet", "tmp.csv", "tmp.msgpack"]:
+        pd_write(df, fn)
+        print(pd_read(fn))
+        remove_path(fn)
+
+    pd_write(df, "tmp.hdf")
+    print(pd_read("tmp.hdf"))
+    remove_path("tmp.hdf")
+
+
+def test_crop_invalid_edges():
+    df = pd.DataFrame(
+        [
+            [np.nan, np.nan, np.nan],
+            ["A", np.nan, "a2"],
+            ["A", "c", "c2"],
+            ["A", "c", "c2"],
+            [np.nan, np.nan, np.nan],
+        ],
+        columns=["col1", "col2", "col3"],
+    )
+
+    print(df)
+    print(crop_invalid_edges(df, "all"))
+    print(crop_invalid_edges(df, "prefix"))
+    print(crop_invalid_edges(df, "suffix"))
+
+
+def test_split_into_sum_groups():
+    df = pd.DataFrame.from_dict({"a": [1.0, 2.0, 3.0], "b": ["<1>", "<2>", "<3>"]})
+    print(split_into_sum_groups(df, [1.0, 1.5, 1.5, 2.0], "a", "group"))
+
+
 if __name__ == "__main__":
-    test()
-    test_tree()
+    # test_merge()
+    # test_tree()
+    # test_read_write()
+    # test_crop_invalid_edges()
+    # test_split_into_sum_groups()
+    test_loc_non_inclusive()
